// MainMenuUI.h: interface for the CMainMenuUI class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINMENUUI_H__54270941_B805_11D6_BD14_A5402071EB3A__INCLUDED_)
#define AFX_MAINMENUUI_H__54270941_B805_11D6_BD14_A5402071EB3A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "AbstractUIElement.h"
#include "MenuItem.h"
#include <D3DX8.h>
#include <vector>

using namespace std;

#define NUM_PART_TEXTURES 3

class CMainMenuUI : public CAbstractUIElement  
{
public:
	CMainMenuUI();
	virtual ~CMainMenuUI();

    //User Input Functions
    virtual void OnMouseMove(const POINT& mousepos, const TransformMatrices* transmats);
    virtual void OnLButtonDown(const POINT& mousepos, const TransformMatrices* transmats);
    virtual void OnLButtonUp(const POINT& mousepos, const TransformMatrices* transmats);

    //Drawing and Animation Functions
    virtual HRESULT Render(const D3DXMATRIX &matWorld);
    virtual HRESULT FrameMove(const POINT& mousepos, const TransformMatrices* transmats);

    //DirectX Device Object Functions
    virtual HRESULT InitDeviceObjects(LPDIRECT3DDEVICE8 pd3dDevice);
    virtual HRESULT RestoreDeviceObjects();
    virtual HRESULT InvalidateDeviceObjects();
    virtual HRESULT DeleteDeviceObjects();

protected:

    void    RenderParticleRing(const D3DXMATRIX &matWorld, BOOL bVert);

    LPDIRECT3DDEVICE8           m_pd3dDevice;           // A D3DDevice used for rendering
    ID3DXMesh*                  m_pD3DXMeshSphere;      // D3DX mesh to store sphere
    D3DXVECTOR3                 m_vMenuPos;             // Position of the menu

    //member variables to draw items
    FLOAT                       m_fItemRot;             // Rotation Amt of main ring
    INT                         m_iSelectedItem;        // ID of item selected, -1 if none
    FLOAT                       m_fSelectItemRot;       // Rotation Amt of selected item ring
    vector<CMenuItem*>          m_vectItems;            // Item UI objects

    //member variables to draw the particle system
    D3DVECTOR*                  m_pParticles;           // Particles for the flames
    LPDIRECT3DVERTEXBUFFER8     m_pParticleVB;          // Vertex buffer for rendering a particle
    LPDIRECT3DTEXTURE8          m_pParticleTextures[NUM_PART_TEXTURES];
    INT                         m_iCurTexture;          // Current texture for the particle system 

    //member variables for state of menu and menu animation
    enum
    {
        MENU_MINIMIZED = 0,
        MENU_MINTOMAX  = 1,
        MENU_MAXIMIZED = 2,
        MENU_MAXTOMIN  = 3,

        MENU_MOUSEDOWN = 4,
        MENU_DRAGGING  = 5
    };
    DWORD                   m_dwMode;               // Whether the menu is minimized or not
    INT                     m_iAnimFrame;           // Frame within an animation mode
};

#endif // !defined(AFX_MAINMENUUI_H__54270941_B805_11D6_BD14_A5402071EB3A__INCLUDED_)
