// MenuItem.cpp: implementation of the CMenuItem class.
//
//////////////////////////////////////////////////////////////////////

#include "MenuItem.h"
#include "dxutil.h"
#include "d3dutil.h"


PARTICLEVERTEX g_ItemVertices[] =
{
    //     x,      y,     z,       color,     tu,    tv        
    {  -0.3f,  -0.3f,  0.0f,  0xffffffff,   0.0f,  1.0f}, 
    {  -0.3f,   0.3f,  0.0f,  0xffffffff,   0.0f,  0.0f},
    {   0.3f,  -0.3f,  0.0f,  0xffffffff,   1.0f,  1.0f},
    {   0.3f,   0.3f,  0.0f,  0xffffffff,   1.0f,  0.0f},
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMenuItem::CMenuItem() : m_pd3dDevice(NULL), m_pItemTexture(NULL), m_pItemVB(NULL)
{

}

CMenuItem::~CMenuItem()
{
    DeleteDeviceObjects();
}

//returns whether the screen coords are in bounds of the item
BOOL CMenuItem::IsInBounds(const POINT& mousepos, const TransformMatrices* transmats)
{
    D3DXVECTOR3 vScreen;
    D3DXVECTOR3 vOrigin = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
    D3DXVec3Project(&vScreen, &vOrigin, &transmats->viewport,
                    &transmats->matProj, &transmats->matView, &transmats->matWorld);
    return ( ((mousepos.x - vScreen.x) * (mousepos.x - vScreen.x) + 
              (mousepos.y - vScreen.y) * (mousepos.y - vScreen.y)) < 400 );
}

void CMenuItem::OnMouseMove(const POINT& mousepos)
{
}

void CMenuItem::OnLButtonDown(const POINT& mousepos)
{
}

void CMenuItem::OnLButtonUp(const POINT& mousepos)
{
}

HRESULT CMenuItem::Render(const D3DXMATRIX &matWorld)
{
    D3DXMATRIX matBillboard = matWorld;
    m_pd3dDevice->SetStreamSource( 0, m_pItemVB, sizeof(PARTICLEVERTEX) );
    m_pd3dDevice->SetTexture( 0, m_pItemTexture );

    matBillboard._11 = matBillboard._22 = matBillboard._33 = 0.5f;
    matBillboard._12 = matBillboard._13 = matBillboard._21 = 0.0f;
    matBillboard._23 = matBillboard._31 = matBillboard._32 = 0.0f;

    m_pd3dDevice->SetTransform(D3DTS_WORLD, &matBillboard);

    // Render the billboard
    m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 2 );

    return S_OK;
}

HRESULT CMenuItem::FrameMove(const POINT& mousepos, const TransformMatrices* transmats)
{
    return S_OK;
}

HRESULT CMenuItem::InitDeviceObjects(LPDIRECT3DDEVICE8 pd3dDevice)
{
    m_pd3dDevice = pd3dDevice;

    // Create a quad for rendering each particle
    if( FAILED( m_pd3dDevice->CreateVertexBuffer( sizeof(g_ItemVertices),
                                                  D3DUSAGE_WRITEONLY, D3DFVF_PARTICLEVERTEX,
                                                  D3DPOOL_MANAGED, &m_pItemVB ) ) )
    {
        return E_FAIL;
    }

    // Copy particle mesh data into vertexbuffer
    PARTICLEVERTEX* vBuf;
    m_pItemVB->Lock( 0, 0, (BYTE**)&vBuf, 0 );
    memcpy(vBuf, g_ItemVertices, sizeof(g_ItemVertices)); 
    m_pItemVB->Unlock();

    if( FAILED( D3DUtil_CreateTexture( m_pd3dDevice, "item0.tga", &m_pItemTexture ) ) )
        return E_FAIL;

    return S_OK;
}

HRESULT CMenuItem::RestoreDeviceObjects()
{
    return S_OK;
}

HRESULT CMenuItem::InvalidateDeviceObjects()
{
    return S_OK;
}

HRESULT CMenuItem::DeleteDeviceObjects()
{
    SAFE_RELEASE( m_pItemVB );
    SAFE_RELEASE( m_pItemTexture );
    m_pd3dDevice = NULL;
    return S_OK;
}